package pps.core.common.utils;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.UUID;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.IdUtil;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import xstartup.base.XStartup;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 封装工具类
 *
 * @author ZWT
 * @date 2023/08/25 16:08
 */
public class BaseUtils {

    /**
     * 按每500个一组分割
     */
    public static final Integer BATCH_SIZE = 500;
    /**
     * 数据库类型
     */
    private static final String DATA_BASE_ID = XStartup.getCurrent().getProperty("x.db.databaseId");

    /**
     * 获取唯一一条
     *
     * @return {@link String }
     */
    public static String getLastLimit() {
        if (CharSequenceUtil.equals(DATA_BASE_ID, BusinessConstant.ORACLE)) {
            return BusinessConstant.LAST_LIMIT_ORACLE;
        }
        return BusinessConstant.LAST_LIMIT;
    }

    /**
     * 计算批次数
     */
    public static Integer countStep(Integer totalSize) {
        return (totalSize + BATCH_SIZE - 1) / BATCH_SIZE;
    }

    /**
     * 分批处理
     *
     * @param list 列表
     * @return {@link List}<{@link List}<{@link E}>>
     */
    public static <E> List<List<E>> getSubList(List<E> list) {
        return Stream.iterate(0, n -> n + 1)
                .limit(countStep(list.size()))
                .parallel()
                .map(a -> list.stream().skip((long) a * BATCH_SIZE)
                        .limit(BATCH_SIZE).parallel()
                        .collect(Collectors.toList()))
                .collect(Collectors.toList());
    }

    /**
     * 设置实体默认值
     *
     * @param model   实体
     * @param session 会话
     */
    public static void setBaseModelDefault(BaseModel model, PpsUserSession session) {
        String id = model.getId();
        Date now = new Date();
        String userId = session.getId();
        String userName = session.getUserName();
        if (CharSequenceUtil.isEmpty(id)) {
            model.setCreateById(userId);
            model.setCreateByName(userName);
            model.setCreateTime(now);
            model.setIsDeleted(BusinessConstant.ONE);
        }
        model.setModifyById(userId);
        model.setModifyByName(userName);
        model.setModifyTime(now);
    }

    /**
     * 创建基础模型默认值
     *
     * @param model   模型
     * @param session 一场
     */
    public static void createBaseModelDefault(BaseModel model, PpsUserSession session) {
        Date now = DateUtil.date();
        model.setId(IdUtil.simpleUUID());
        model.setIsDeleted(1);
        String userId = session.getId();
        String userName = session.getUserName();
        model.setCreateById(userId);
        model.setCreateByName(userName);
        model.setCreateTime(now);
        model.setModifyById(userId);
        model.setModifyByName(userName);
        model.setModifyTime(now);
    }

    /**
     * 设置实体默认值(定时任务)
     *
     * @param model 模型
     */
    public static void setBaseModelDefaultForJob(BaseModel model) {
        DateTime beginOfDay = DateUtil.beginOfDay(DateTime.now());
        model.setId(BaseUtils.randomUUIDString());
        model.setIsDeleted(1);
        model.setCreateById("Task");
        model.setCreateByName("Task");
        model.setCreateTime(beginOfDay);
        model.setModifyById("Task");
        model.setModifyByName("Task");
        model.setModifyTime(beginOfDay);
    }

    /**
     * 获取周执行周期
     *
     * @param date 日期
     * @return {@link String}
     */
    public static String getExecutionCycleForWeek(DateTime date) {
        return date.year() + "年第" + date.weekOfYear() + '周';
    }

    /**
     * 获取校准执行周期
     *
     * @param date   日期
     * @param offset 抵消
     * @return {@link String}
     */
    public static String getExecutionCycleForCalibration(Date date, int offset) {
        return DateUtil.format(date, BusinessConstant.DATE_FORMAT_DAY_C) + '-' + DateUtil.format(DateUtil.offsetDay(date, offset - 1), BusinessConstant.DATE_FORMAT_DAY_C);
    }

    /**
     * 获取校准执行周期
     *
     * @param startDate 开始日期
     * @param endDate   结束日期
     * @return {@link String}
     */
    public static String getExecutionCycleForCalibration(Date startDate, Date endDate) {
        return DateUtil.format(startDate, BusinessConstant.DATE_FORMAT_DAY_C) + '-' + DateUtil.format(endDate, BusinessConstant.DATE_FORMAT_DAY_C);
    }

    /**
     * 获取时差分钟
     *
     * @param optimizeTime 优化后时间
     * @param originalTime 原始时间
     * @return int
     */
    public static int getTimeDifferenceMinute(DateTime optimizeTime, DateTime originalTime) {
        int between = (int) optimizeTime.between(originalTime, DateUnit.MINUTE);
        if (optimizeTime.compareTo(originalTime) < 0) {
            between = ~between + 1;
        }
        return between;
    }

    /**
     * 获取结束时间字符串
     *
     * @param endTimeOptimize 结束时间优化
     * @return {@link String}
     */
    public static String getEndTimeString(DateTime endTimeOptimize) {
        String endTimeString;
        if (endTimeOptimize.compareTo(BusinessConstant.DATE_FLAG) >= 0) {
            endTimeString = BusinessConstant.END_OF_DAY_TIME;
        } else {
            endTimeString = endTimeOptimize.toString(BusinessConstant.MINUTES_FORMAT);
        }
        return endTimeString;
    }

    /**
     * 获取月份字符串
     *
     * @param monthNum 月份
     * @return {@link String}
     */
    public static String getMonthString(int monthNum) {
        String month;
        if (monthNum < BusinessConstant.TEN) {
            month = "0" + monthNum;
        } else {
            month = String.valueOf(monthNum);
        }
        return month;
    }

    /**
     * 除法
     *
     * @param one 一
     * @param two 二
     * @return {@link BigDecimal }
     */
    public static BigDecimal divide(BigDecimal one, BigDecimal two) {
        BigDecimal result;
        if (two.compareTo(BigDecimal.ZERO) > 0) {
            result = one.divide(two, 2, RoundingMode.HALF_UP);
        } else {
            result = BigDecimal.ZERO;
        }
        return result;
    }

    /**
     * 计算率
     * 示例:
     * 消纳率:就地消纳量/累计用电*100%
     *
     * @param one 一
     * @param two 二
     * @return {@link BigDecimal }
     */
    public static BigDecimal getRate(BigDecimal one, BigDecimal two) {
        BigDecimal rate;
        if (two.compareTo(BigDecimal.ZERO) > 0) {
            rate = one.multiply(BusinessConstant.BIG_DECIMAL_100)
                    .divide(two, 2, RoundingMode.HALF_UP);
        } else {
            rate = BigDecimal.ZERO;
        }
        return rate;
    }

    /**
     * 用能成本
     * <p>
     * 光伏发电够抽油机运行的时段:度电成本*抽油机运行功率*运行时长(h)
     * 光伏发电不够抽油机运行时段，该时段的电费(峰谷平不同，每月也有区别)*抽油机运行功率*运行时长(h)
     * 降低费用为优化前-优化后
     *
     * @param one   电费
     * @param two   运行功率
     * @param three 运行时长
     * @return {@link BigDecimal }
     */
    public static BigDecimal energyCost(BigDecimal one, BigDecimal two, BigDecimal three) {
        return one.multiply(two)
                .multiply(three)
                .setScale(2, RoundingMode.UP);
    }

    /**
     * 大于0
     *
     * @param one 一
     * @return boolean
     */
    public static boolean greaterThan_0(BigDecimal one) {
        return one.compareTo(BigDecimal.ZERO) > 0;
    }

    /**
     * 获取环比
     * (本期-上期) /上期*100%
     *
     * @param currentPeriod 本期
     * @param lastPeriod    上期
     * @return {@link BigDecimal}
     */
    public static BigDecimal getCycleRatio(BigDecimal currentPeriod, BigDecimal lastPeriod) {
        BigDecimal cycleRatio;
        if (lastPeriod.compareTo(BigDecimal.ZERO) > 0) {
            cycleRatio = currentPeriod.subtract(lastPeriod)
                    .multiply(BusinessConstant.BIG_DECIMAL_100)
                    .divide(lastPeriod, 2, RoundingMode.HALF_UP);
        } else {
            cycleRatio = BigDecimal.ZERO;
        }
        return cycleRatio;
    }

    /**
     * 获取uuid字符串
     *
     * @return {@link String }
     */
    public static String randomUUIDString() {
        return UUID.randomUUID(true).toString(true);
    }
}