package pps.core.base.excel;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import pps.core.base.entity.WindPredictionFutureView;
import pps.core.base.entity.WindPredictionHistoryView;
import pps.core.base.mapper.WindPredictionFutureViewMapper;
import pps.core.base.mapper.WindPredictionHistoryViewMapper;
import pps.core.base.service.data.base_excel.WindPredictionExcelData;
import pps.core.common.utils.BaseUtils;
import xstartup.base.XContext;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

/**
 * 风资源数据(测试用)
 *
 * @author ZWT
 * @date 2024/08/30 15:46
 */
public class WindPredictionDataListener implements ReadListener<WindPredictionExcelData> {

    private XContext context;
    private String stationId;
    /**
     * 缓存的数据
     */
    private List<WindPredictionExcelData> cachedDataList = new ArrayList<>(4096);

    public WindPredictionDataListener(XContext context, String stationId) {
        this.context = context;
        this.stationId = stationId;

    }

    @Override
    public void invoke(WindPredictionExcelData windPredictionExcelData, AnalysisContext analysisContext) {
        cachedDataList.add(windPredictionExcelData);
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        this.saveWindPredictionHistory(context, cachedDataList, stationId);
    }

    public void saveWindPredictionHistory(XContext context, List<WindPredictionExcelData> cachedDataList, String stationId) {
        if (CollUtil.isNotEmpty(cachedDataList)) {
            List<WindPredictionFutureView> futureList = new ArrayList<>(cachedDataList.size());
            List<WindPredictionHistoryView> historyList = new ArrayList<>(cachedDataList.size());
            //按日期顺序排序
            cachedDataList.sort(Comparator.comparing(WindPredictionExcelData::getDataTime));
            //数据分界线
            DateTime futureFlag = DateUtil.offsetDay(DateUtil.beginOfDay(CollUtil.getLast(cachedDataList).getDataTime()), -20);
            //数据分片
            for (WindPredictionExcelData excelData : cachedDataList) {
                if (DateUtil.compare(excelData.getDataTime(), futureFlag) >= 0) {
                    //未来数据
                    WindPredictionFutureView build = WindPredictionFutureView.builder()
                            .stationId(stationId)
                            .dataTime(excelData.getDataTime())
                            .windDirection(excelData.getWindDirection())
                            .windSpeed(excelData.getWindSpeed())
                            .airTemperature(excelData.getAirTemperature())
                            .humidity(excelData.getHumidity())
                            .pressure(excelData.getPressure())
                            .actualWindSpeed(excelData.getActualWindSpeed())
                            .actualPower(excelData.getActualPower())
                            .predictedPower(excelData.getPredictedPower())
                            .build();
                    //去重
                    if (futureList.stream()
                            .noneMatch(item -> DateUtil.compare(item.getDataTime(), build.getDataTime()) == 0)) {
                        build.setId(BaseUtils.randomUUIDString());
                        futureList.add(build);
                    }
                } else {
                    //历史数据
                    WindPredictionHistoryView build = WindPredictionHistoryView.builder()
                            .stationId(stationId)
                            .dataTime(excelData.getDataTime())
                            .windDirection(excelData.getWindDirection())
                            .windSpeed(excelData.getWindSpeed())
                            .airTemperature(excelData.getAirTemperature())
                            .humidity(excelData.getHumidity())
                            .pressure(excelData.getPressure())
                            .actualWindSpeed(excelData.getActualWindSpeed())
                            .actualPower(excelData.getActualPower())
                            .predictedPower(excelData.getPredictedPower())
                            .build();
                    //去重
                    if (historyList.stream()
                            .noneMatch(item -> DateUtil.compare(item.getDataTime(), build.getDataTime()) == 0)) {
                        build.setId(BaseUtils.randomUUIDString());
                        historyList.add(build);
                    }
                }
            }
            //记录
            String futureStart = DateUtil.formatDateTime(CollUtil.getFirst(futureList).getDataTime());
            String futureEnd = DateUtil.formatDateTime(CollUtil.getLast(futureList).getDataTime());
            String historyStart = DateUtil.formatDateTime(CollUtil.getFirst(historyList).getDataTime());
            String historyEnd = DateUtil.formatDateTime(CollUtil.getLast(historyList).getDataTime());
            //入库
            if (CollUtil.isNotEmpty(futureList)) {
                WindPredictionFutureViewMapper futureMapper = context.getBean(WindPredictionFutureViewMapper.class);
                if (futureList.size() > BaseUtils.BATCH_SIZE) {
                    List<List<WindPredictionFutureView>> subList = BaseUtils.getSubList(futureList);
                    subList.forEach(futureMapper::insertBatch);
                } else {
                    futureMapper.insertBatch(futureList);
                }
            }
            if (CollUtil.isNotEmpty(historyList)) {
                WindPredictionHistoryViewMapper historyMapper = context.getBean(WindPredictionHistoryViewMapper.class);
                if (historyList.size() > BaseUtils.BATCH_SIZE) {
                    List<List<WindPredictionHistoryView>> subList = BaseUtils.getSubList(historyList);
                    subList.forEach(historyMapper::batchInsert);
                } else {
                    historyMapper.batchInsert(historyList);
                }
            }
        }
    }
}
