package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.base.service.IWindPredictionHistoryCloudService;
import pps.core.base.entity.ThirdWindPowerGenerationEnt;
import pps.core.base.entity.WindPredictionFutureEnt;
import pps.core.base.entity.WindPredictionHistoryView;
import pps.core.base.mapper.ThirdWindPowerGenerationMapper;
import pps.core.base.mapper.WindPredictionFutureMapper;
import pps.core.base.mapper.WindPredictionHistoryViewMapper;
import pps.core.base.utils.ServiceUtil;
import pps.core.common.utils.BaseUtils;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 风资源训练集数据Cloud模块
 *
 * @author ZWT
 * @date 2024/08/30 14:07
 */
@XService
public class WindPredictionHistoryCloudServiceImpl implements IWindPredictionHistoryCloudService {

    /**
     * 风资源训练集定时任务
     * wind_prediction_history的 actual_power存实际功率
     * wind_prediction_future的 actual_power为预测功率
     *
     * @param context 上下文
     * @return {@link XServiceResult }
     */
    @Override
    public XServiceResult windPredictionHistoryJob(XContext context) {
        DateTime startTime = DateUtil.beginOfDay(DateUtil.yesterday());
        DateTime endTime = DateUtil.endOfDay(startTime);
        //取日期范围内已预测数据
        WindPredictionFutureMapper mapper = context.getBean(WindPredictionFutureMapper.class);
        List<WindPredictionFutureEnt> list = mapper.selectList(new QueryWrapper<WindPredictionFutureEnt>()
                .select("station_id",
                        "data_time",
                        "wind_direction",
                        "wind_speed",
                        "air_temperature",
                        "humidity",
                        "pressure",
                        "actual_wind_speed",
                        "actual_power AS predicted_power")
                .lambda()
                .between(WindPredictionFutureEnt::getDataTime, startTime, endTime)
        );
        if (CollUtil.isNotEmpty(list)) {
            //查实际发电数据
            ThirdWindPowerGenerationMapper generationMapper = context.getBean(ThirdWindPowerGenerationMapper.class);
            List<ThirdWindPowerGenerationEnt> generationList = generationMapper.selectList(new QueryWrapper<ThirdWindPowerGenerationEnt>()
                    .select("station_id", "collect_time", "MAX( actual_power ) AS actual_power",
                            "MAX( actual_generation ) AS actual_generation", "MAX( actual_wind_speed ) AS actual_wind_speed",
                            "MAX( actual_wind_direction ) AS actual_wind_direction")
                    .lambda()
                    .between(ThirdWindPowerGenerationEnt::getCollectTime, startTime, endTime)
                    .eq(ThirdWindPowerGenerationEnt::getSystemSource, ServiceUtil.getOilFieldCode(context))
                    .groupBy(ThirdWindPowerGenerationEnt::getStationId, ThirdWindPowerGenerationEnt::getCollectTime)
            );
            Map<String, Map<Date, ThirdWindPowerGenerationEnt>> collect;
            if (CollUtil.isEmpty(generationList)) {
                collect = Collections.emptyMap();
            } else {
                collect = generationList.stream()
                        .collect(Collectors.groupingBy(ThirdWindPowerGenerationEnt::getStationId, Collectors.toMap(ThirdWindPowerGenerationEnt::getCollectTime, Function.identity())));
            }
            //数据处理
            List<WindPredictionHistoryView> historyList = this.dataProcess(list, collect);
            //按时间范围删除数据
            WindPredictionHistoryViewMapper viewMapper = context.getBean(WindPredictionHistoryViewMapper.class);
            viewMapper.deleteByTimeRange(startTime, endTime);
            if (historyList.size() > BaseUtils.BATCH_SIZE) {
                List<List<WindPredictionHistoryView>> subList = BaseUtils.getSubList(historyList);
                subList.forEach(viewMapper::batchInsert);
            } else {
                viewMapper.batchInsert(historyList);
            }
        }
        return XServiceResult.OK;
    }

    /*------------------------- private -------------------------*/

    /**
     * 数据处理
     * todo 模拟数据
     *
     * @param list    列表
     * @param collect 收集
     * @return {@link List }<{@link WindPredictionHistoryView }>
     */
    private List<WindPredictionHistoryView> dataProcess(List<WindPredictionFutureEnt> list, Map<String, Map<Date, ThirdWindPowerGenerationEnt>> collect) {
        List<WindPredictionHistoryView> historyList = new ArrayList<>(list.size());
        Map<Date, ThirdWindPowerGenerationEnt> map;
        ThirdWindPowerGenerationEnt ent;
        for (WindPredictionFutureEnt future : list) {
            WindPredictionHistoryView view = new WindPredictionHistoryView();
            view.setId(BaseUtils.randomUUIDString());
            view.setStationId(future.getStationId());
            view.setDataTime(future.getDataTime());
            view.setWindDirection(future.getWindDirection());
            view.setWindSpeed(future.getWindSpeed());
            view.setAirTemperature(future.getAirTemperature());
            view.setHumidity(future.getHumidity());
            view.setPressure(future.getPressure());
            view.setPredictedPower(future.getPredictedPower());
            //匹配真实数据
            boolean flag = true;
            if (collect.containsKey(future.getStationId())) {
                map = collect.get(future.getStationId());
                if (map.containsKey(future.getDataTime())) {
                    flag = false;
                    ent = map.get(future.getDataTime());
                    view.setActualPower(ent.getActualPower());
                    view.setActualWindSpeed(ent.getActualWindSpeed());
                    view.setActualWindDirection(ent.getAccurateWindDirection());
                }
            }
            if (flag) {
                view.setActualPower(BigDecimal.ZERO);
                view.setActualWindSpeed(BigDecimal.ZERO);
                view.setActualWindDirection(BigDecimal.ZERO);
            }
            historyList.add(view);
        }
        return historyList;
    }
}