package pps.core.space.utils;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.IBasePriceStrategyCloudService;
import pps.cloud.base.service.IBaseWellheadCloudService;
import pps.cloud.base.service.IConfigOilFieldCloudService;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.base.service.data.base_power_line.GetBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.GetBasePowerLineViewOutput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantInput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantViewOutput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailInput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailOutput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadInput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadOutput;
import pps.cloud.base.service.data.config_oil_field.GetConfigOilFieldOutput;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.sys_dictionary.QuerySysDictionaryViewInput;
import pps.cloud.system.service.data.sys_dictionary.QuerySysDictionaryViewOutput;
import pps.cloud.system.service.data.sys_organization.GetOuListByOuIdsInput;
import pps.cloud.system.service.data.sys_organization.GetOuListTreeOutput;
import pps.cloud.system.service.data.sys_organization.GetSysOrganizationViewInput;
import pps.cloud.system.service.data.sys_organization.GetSysOrganizationViewOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.space.service.data.SpaceOptimizeDateDuration;
import pps.core.space.service.data.SpaceOptimizeDateOutput;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 常用服务调用工具类
 *
 * @author ZWT
 * @date 2023/09/05
 */
public class ServiceUtil {

    /*-------------------------服务调用-------------------------*/

    /**
     * 获取基础井列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link List }<{@link DynamicQueryBaseWellheadOutput }>
     */
    public static List<DynamicQueryBaseWellheadOutput> getBaseWellList(XContext context, DynamicQueryBaseWellheadInput input) {
        IBaseWellheadCloudService cloudService = context.getBean(IBaseWellheadCloudService.class);
        XListResult<DynamicQueryBaseWellheadOutput> result = cloudService.queryBaseWellheadListByParam(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获取输电线电站列表
     *
     * @param context 上下文
     * @param input   入参
     * @return {@link List}<{@link DynamicQueryBasePowerLinePlantViewOutput}>
     */
    public static List<DynamicQueryBasePowerLinePlantViewOutput> getPowerLinePlantList(XContext context, DynamicQueryBasePowerLinePlantInput input) {
        IBasePowerLineCloudService service = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLinePlantViewOutput> result = service.getPowerLinePlantViewList(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 通过字典类型查询字典
     *
     * @param context 上下文
     * @param alias   别名
     * @return {@link Map}<{@link String}, {@link String}>
     */
    public static Map<String, String> getDictMap(XContext context, String alias) {
        QuerySysDictionaryViewInput dictInput = new QuerySysDictionaryViewInput();
        dictInput.setAlias(alias);
        dictInput.setPage(1);
        dictInput.setLimit(999);
        SystemDictionaryService dictionaryService = context.getBean(SystemDictionaryService.class);
        XListResult<QuerySysDictionaryViewOutput> dictPageResult = dictionaryService.queryChildSysDictionarysByParentAlias(context, dictInput);
        dictPageResult.throwIfFail();
        return dictPageResult.getResult().stream()
                .collect(Collectors.toMap(QuerySysDictionaryViewOutput::getDicKey, QuerySysDictionaryViewOutput::getDicName));
    }

    /**
     * 通过组织id获取系统组织名称
     *
     * @param context 上下文
     * @param ouId    组织id
     * @return {@link String}
     */
    public static String getSysOrgNameById(XContext context, String ouId) {
        SysOrganizationCloudService orgCloudService = context.getBean(SysOrganizationCloudService.class);
        GetSysOrganizationViewInput viewInput = new GetSysOrganizationViewInput();
        viewInput.setId(ouId);
        XSingleResult<GetSysOrganizationViewOutput> organization = orgCloudService.getSysOrganizationById(context, viewInput);
        organization.throwIfFail();
        return Optional.ofNullable(organization.getResult())
                .map(GetSysOrganizationViewOutput::getOuName)
                .orElse(null);
    }

    /**
     * 通过组织机构ID集合获取组织名称
     *
     * @param context 上下文
     * @param ouIds   ou id
     * @return {@link Map}<{@link String}, {@link String}>
     */
    public static Map<String, String> getOuMapByOuIds(XContext context, List<String> ouIds) {
        GetOuListByOuIdsInput input = new GetOuListByOuIdsInput();
        input.setOuIdList(ouIds);
        SysOrganizationCloudService organizationCloudService = context.getBean(SysOrganizationCloudService.class);
        XListResult<GetOuListTreeOutput> ouListByOuIds = organizationCloudService.getOuListByOuIds(context, input);
        ouListByOuIds.throwIfFail();
        return ouListByOuIds.getResult().stream()
                .collect(Collectors.toMap(
                        GetOuListTreeOutput::getOuId, GetOuListTreeOutput::getOuName
                ));
    }

    /**
     * 通过ID查询线路名称
     *
     * @param context 上下文
     * @param lineId  线路id
     * @return {@link String}
     */
    public static String getPowerLineNameById(XContext context, String lineId) {
        IBasePowerLineCloudService lineService = context.getBean(IBasePowerLineCloudService.class);
        XSingleResult<GetBasePowerLineViewOutput> result = lineService.getBasePowerLineById(context, GetBasePowerLineInput.builder()
                .id(lineId)
                .build());
        result.throwIfFail();
        return Optional.ofNullable(result.getResult())
                .map(GetBasePowerLineViewOutput::getLineName)
                .orElse(null);
    }

    /**
     * 获取电源线列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link List }<{@link DynamicQueryBasePowerLineOutput }>
     */
    public static List<DynamicQueryBasePowerLineOutput> getPowerLineList(XContext context, DynamicQueryBasePowerLineInput input) {
        IBasePowerLineCloudService lineService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineOutput> result = lineService.getBasePowerLineList(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 通过线路ID集合获取线路名称Map
     *
     * @param context 上下文
     * @param lineIds 线路ID
     * @return {@link Map}<{@link String}, {@link String}>
     */
    public static Map<String, String> getPowerLineNameMap(XContext context, List<String> lineIds) {
        return getPowerLineList(context, DynamicQueryBasePowerLineInput.builder()
                .lineIds(lineIds)
                .build()).stream()
                .collect(
                        Collectors.toMap(DynamicQueryBasePowerLineOutput::getId, DynamicQueryBasePowerLineOutput::getLineName)
                );
    }

    /**
     * 条件查询策略配置详情列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link List}<{@link GetBasePriceStrategyDetailOutput}>
     */
    public static List<GetBasePriceStrategyDetailOutput> getStrategyDetailList(XContext context, GetBasePriceStrategyDetailInput input) {
        IBasePriceStrategyCloudService service = context.getBean(IBasePriceStrategyCloudService.class);
        XListResult<GetBasePriceStrategyDetailOutput> result = service.queryStrategyDetailList(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获取优化日期列表
     *
     * @param begin 开始
     * @param end   终止
     * @return {@link List}<{@link SpaceOptimizeDateOutput}>
     */
    public static List<SpaceOptimizeDateOutput> getOptimizeDateList(Date begin, Date end) {
        return DateUtil.rangeToList(
                        DateUtil.beginOfDay(begin),
                        DateUtil.endOfDay(DateUtil.offsetDay(end, -1)),
                        DateField.DAY_OF_YEAR).stream()
                .map(r -> {
                    SpaceOptimizeDateOutput spaceOptimizeDateOutput = new SpaceOptimizeDateOutput();
                    spaceOptimizeDateOutput.setDateTime(r);
                    spaceOptimizeDateOutput.setOptimizeCode(BusinessConstant.ZERO);
                    spaceOptimizeDateOutput.setOptimizeValue(BusinessConstant.OPTIMIZED_C);
                    return spaceOptimizeDateOutput;
                }).collect(Collectors.toList());
    }

    /**
     * 封装全天时间段
     *
     * @param optimizeList 优化列表
     * @return {@link List}<{@link SpaceOptimizeDateDuration}>
     */
    public static List<SpaceOptimizeDateDuration> getAllDateDuration(List<SpaceOptimizeDateDuration> optimizeList) {
        List<SpaceOptimizeDateDuration> allList = new ArrayList<>(16);
        if (CollUtil.isEmpty(optimizeList)) {
            allList.add(
                    SpaceOptimizeDateDuration.builder()
                            .openWellTime(BusinessConstant.START_OF_DAY_TIME)
                            .closeWellTime(BusinessConstant.END_OF_DAY_TIME)
                            .isOpen(BusinessConstant.ZERO)
                            .build()
            );
            return allList;
        }
        int size = optimizeList.size();
        for (int i = 0; i < size; i++) {
            SpaceOptimizeDateDuration optimize = optimizeList.get(i);
            String openWellTime = optimize.getOpenWellTime();
            String closeWellTime = optimize.getCloseWellTime();
            optimize.setIsOpen(BusinessConstant.ONE);
            if (i == 0) {
                //如果开始时间不是从00:00开始,初始化开始时间为关井时间
                if (!CharSequenceUtil.equals(BusinessConstant.START_OF_DAY_TIME, openWellTime)) {
                    allList.add(
                            SpaceOptimizeDateDuration.builder()
                                    .openWellTime(BusinessConstant.START_OF_DAY_TIME)
                                    .closeWellTime(openWellTime)
                                    .isOpen(BusinessConstant.ZERO)
                                    .build()
                    );
                }
            }
            //将当前时段添加到新集合
            if (!CharSequenceUtil.equals(openWellTime, closeWellTime)) {
                allList.add(optimize);
            }
            //判断当前时段到下一时段间是否有关井时段
            int next = i + 1;
            int last = size - 1;
            if (next <= last) {
                SpaceOptimizeDateDuration nextOptimize = optimizeList.get(next);
                String nextOpenWellTime = nextOptimize.getOpenWellTime();
                if (!CharSequenceUtil.equals(closeWellTime, nextOpenWellTime)) {
                    allList.add(
                            SpaceOptimizeDateDuration.builder()
                                    .openWellTime(closeWellTime)
                                    .closeWellTime(nextOpenWellTime)
                                    .isOpen(BusinessConstant.ZERO)
                                    .build()
                    );
                }
            }
            if (i == last) {
                //如果结束时间不是24:00,初始化最终关井时间
                if (!CharSequenceUtil.equals(BusinessConstant.END_OF_DAY_TIME, closeWellTime)) {
                    allList.add(
                            SpaceOptimizeDateDuration.builder()
                                    .openWellTime(closeWellTime)
                                    .closeWellTime(BusinessConstant.END_OF_DAY_TIME)
                                    .isOpen(BusinessConstant.ZERO)
                                    .build()
                    );
                }
            }
        }
        return allList;
    }

    /**
     * 检查大间开是否开井
     *
     * @param openWellDay  开井日
     * @param closeWellDay 闭井日
     * @param createTime   创建时间
     * @param currentTime  当前时间
     * @return boolean
     */
    public static boolean checkWellIsOpen(Integer openWellDay, Integer closeWellDay, Date createTime, DateTime currentTime) {
        boolean openFlag = true;
        //计算开关井周期总天数
        openWellDay = Objects.isNull(openWellDay) ? 0 : openWellDay;
        closeWellDay = Objects.isNull(closeWellDay) ? 0 : closeWellDay;
        Integer totalDays = openWellDay + closeWellDay;
        if (!totalDays.equals(0)) {
            //获取间开井口创建时间和当天的时间差
            int betweenDay = (int) DateUtil.betweenDay(createTime, currentTime, true);
            switch (totalDays.compareTo(betweenDay)) {
                case -1:
                    //按周期取余数和开井时间比较,超过开井时间就是关井
                    if (openWellDay.compareTo(betweenDay % totalDays) >= 0) {
                        //开井
                        openFlag = true;
                    } else {
                        //关井
                        openFlag = false;
                    }
                    break;
                case 0:
                    //相等为周期最后一天,只要关井天数不为0就是关井
                    if (closeWellDay.equals(0)) {
                        //开井
                        openFlag = true;
                    } else {
                        //关井
                        openFlag = false;
                    }
                    break;
                case 1:
                    //当前未过一个执行周期,判断开井时间是否大于时差,大于就是开井
                    if (openWellDay.compareTo(betweenDay) >= 0) {
                        //开井
                        openFlag = true;
                    } else {
                        //关井
                        openFlag = false;
                    }
                    break;
                default:
            }
        } else {
            //关井
            openFlag = false;
        }
        return openFlag;
    }

    /**
     * 获取当前配置
     *
     * @param context 上下文
     * @return {@link GetConfigOilFieldOutput }
     */
    public static GetConfigOilFieldOutput getCurrentConfig(XContext context) {
        IConfigOilFieldCloudService bean = context.getBean(IConfigOilFieldCloudService.class);
        XSingleResult<GetConfigOilFieldOutput> currentConfig = bean.getCurrentConfig(context);
        currentConfig.throwIfFail();
        return currentConfig.getResult();
    }

    /**
     * 获取油田代码
     *
     * @param context 上下文
     * @return {@link String }
     */
    public static String getOilFieldCode(XContext context) {
        //判断部署环境
        GetConfigOilFieldOutput currentConfig = getCurrentConfig(context);
        return ObjectUtil.defaultIfNull(currentConfig.getOilFieldCode(), BusinessConstant.DEFAULT_VALUE);
    }
}