package pps.core.prediction.service;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.prediction.service.IThirdPowerCloudService;
import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerInput;
import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerOutput;
import pps.cloud.prediction.service.data.third_daily_accumulation_update.GetThirdDailyAccumulationUpdateInput;
import pps.cloud.prediction.service.data.third_daily_accumulation_update.GetThirdDailyAccumulationUpdateOutput;
import pps.cloud.prediction.service.data.wellhead_daily_production_situation.GetWellheadDailyProductionSituationInput;
import pps.cloud.prediction.service.data.wellhead_daily_production_situation.GetWellheadDailyProductionSituationOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.prediction.entity.ThirdActivePowerView;
import pps.core.prediction.entity.ThirdDailyAccumulationUpdateEnt;
import pps.core.prediction.entity.WellheadDailyProductionSituationEnt;
import pps.core.prediction.mapper.ThirdActivePowerViewMapper;
import pps.core.prediction.mapper.ThirdDailyAccumulationUpdateMapper;
import pps.core.prediction.mapper.WellheadDailyProductionSituationMapper;
import pps.core.prediction.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;

import java.util.List;

/**
 * 第三方有功功率Cloud模块
 *
 * @author ZWT
 * @date 2024/05/23 14:18
 */
@XText("第三方有功功率Cloud模块")
@XService
public class ThirdPowerCloudServiceImpl implements IThirdPowerCloudService {

    /**
     * 获得15分钟平均功率
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetThirdActivePowerOutput }>
     */
    @Override
    public XListResult<GetThirdActivePowerOutput> queryAvgPhotovoltaicPower(XContext context, GetThirdActivePowerInput input) {
        ThirdActivePowerViewMapper mapper = context.getBean(ThirdActivePowerViewMapper.class);
        List<ThirdActivePowerView> list = mapper.selectAvgPhotovoltaicPower(ThirdActivePowerView.builder()
                .systemSource(input.getSystemSource())
                .createDate(input.getCreateDate())
                .build());
        return XListResult.success(XCopyUtils.copyNewList(list, GetThirdActivePowerOutput.class));
    }

    /**
     * 获得30分钟实际功率
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetThirdActivePowerOutput }>
     */
    @Override
    public XListResult<GetThirdActivePowerOutput> queryPhotovoltaicPower30(XContext context, GetThirdActivePowerInput input) {
        ThirdActivePowerViewMapper mapper = context.getBean(ThirdActivePowerViewMapper.class);
        List<ThirdActivePowerView> list = mapper.selectAvgPower30(ThirdActivePowerView.builder()
                .stationName(input.getStationName())
                .stationNames(input.getStationNames())
                .systemSource(ServiceUtil.getOilFieldCode(context))
                .startTime(input.getCreateDate())
                .endTime(DateUtil.offsetDay(input.getCreateDate(), 1))
                .build());
        return XListResult.success(XCopyUtils.copyNewList(list, GetThirdActivePowerOutput.class));
    }

    /**
     * 查询每日井场日累计数据
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetThirdDailyAccumulationUpdateOutput }>
     */
    @Override
    public XListResult<GetThirdDailyAccumulationUpdateOutput> queryDailyUpdatePower(XContext context, GetThirdDailyAccumulationUpdateInput input) {
        QueryWrapper<ThirdDailyAccumulationUpdateEnt> queryWrapper = new QueryWrapper<>();
        if (CharSequenceUtil.equals(context.getProperty("x.db.databaseId"), BusinessConstant.ORACLE)) {
            queryWrapper.select("STATION_NAME",
                            "NVL( AVG( PHOTOVOLTAIC_POWER ), 0 ) AS PHOTOVOLTAIC_POWER",
                            "NVL( AVG( DAILY_ELECTRICITY_CONSUMPTION ), 0 ) AS DAILY_ELECTRICITY_CONSUMPTION")
                    .apply("TO_CHAR( CREATE_DATE, 'YYYY-MM-DD' ) = {0}", DateUtil.date(input.getCreateDate()).toDateStr());
        } else {
            queryWrapper.select("STATION_NAME",
                            "IFNULL( AVG( PHOTOVOLTAIC_POWER ), 0 ) AS PHOTOVOLTAIC_POWER",
                            "IFNULL( AVG( DAILY_ELECTRICITY_CONSUMPTION ), 0 ) AS DAILY_ELECTRICITY_CONSUMPTION")
                    .apply("DATE( CREATE_DATE ) = {0}", input.getCreateDate());
        }
        ThirdDailyAccumulationUpdateMapper mapper = context.getBean(ThirdDailyAccumulationUpdateMapper.class);
        List<ThirdDailyAccumulationUpdateEnt> list = mapper.selectList(queryWrapper.lambda()
                .gt(ThirdDailyAccumulationUpdateEnt::getPhotovoltaicPower, 0)
                .eq(CharSequenceUtil.isNotBlank(input.getSystemSource()), ThirdDailyAccumulationUpdateEnt::getSystemSource, input.getSystemSource())
                .groupBy(ThirdDailyAccumulationUpdateEnt::getStationName)
        );
        return XListResult.success(XCopyUtils.copyNewList(list, GetThirdDailyAccumulationUpdateOutput.class));
    }

    /**
     * 查询每日井口生产情况
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetWellheadDailyProductionSituationOutput }>
     */
    @Override
    public XListResult<GetWellheadDailyProductionSituationOutput> queryWellProductionList(XContext context, GetWellheadDailyProductionSituationInput input) {
        WellheadDailyProductionSituationMapper mapper = context.getBean(WellheadDailyProductionSituationMapper.class);
        QueryWrapper<WellheadDailyProductionSituationEnt> queryWrapper = new QueryWrapper<>();
        if (CharSequenceUtil.equals(context.getProperty("x.db.databaseId"), BusinessConstant.ORACLE)) {
            queryWrapper.select("WELL_NUMBER",
                    "NVL( PRODUCTION_TIME, 0 ) AS PRODUCTION_TIME",
                    "NVL( DAILY_ELECTRICITY_CONSUMPTION, 0 ) AS DAILY_ELECTRICITY_CONSUMPTION",
                    "NVL( DAILY_LIQUID_PRODUCTION, 0 ) AS DAILY_LIQUID_PRODUCTION");
        } else {
            queryWrapper.select("WELL_NUMBER",
                    "IFNULL( PRODUCTION_TIME, 0 ) AS PRODUCTION_TIME",
                    "IFNULL( DAILY_ELECTRICITY_CONSUMPTION, 0 ) AS DAILY_ELECTRICITY_CONSUMPTION",
                    "IFNULL( DAILY_LIQUID_PRODUCTION, 0 ) AS DAILY_LIQUID_PRODUCTION");
        }
        List<WellheadDailyProductionSituationEnt> list = mapper.selectList(queryWrapper.lambda()
                .eq(WellheadDailyProductionSituationEnt::getDataDate, input.getCreateDate())
                .eq(CharSequenceUtil.isNotBlank(input.getSystemSource()), WellheadDailyProductionSituationEnt::getSystemSource, input.getSystemSource())
        );
        return XListResult.success(XCopyUtils.copyNewList(list, GetWellheadDailyProductionSituationOutput.class));
    }
}